#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from tornado.websocket import WebSocketHandler
import tornado.ioloop
import tornado.web

from random import randint as ri
from numpy.linalg import norm
from numpy import array, array_equal

class SquareWS (WebSocketHandler):
	handlers = []
	colors = [array([0, 0, 0])]
	
	def printColors (self):
		print "Square: current colors:"
		for color in self.colors:
			print '\t' \
			    + self.toHex (color[0]) \
		        + self.toHex (color[1]) \
		        + self.toHex (color[2])
	
	def removeColor (self, color):
		ind = 0
		size = len(self.colors)
		while ind != size and not array_equal (self.colors[ind], color):
			ind += 1
		if ind != size:
			self.colors.pop(ind)
		else:
			raise ValueError('array not found in list.')
	
	def bcast (self, msg):
		for handler in self.handlers:
			if handler != self:
				handler.write_message(msg)
	
	def toHex (self, num):
		ret = hex (num)[2:].rstrip ('L')
		if len(ret) < 2:
			ret = '0' + ret
		return ret
		
	def colorStr (self):
		return self.toHex (self.color[0]) \
		     + self.toHex (self.color[1]) \
		     + self.toHex (self.color[2])

	def colorCheck (self, color):
		ok = True
		for tcolor in self.colors:
			dist = abs (norm ( (tcolor - color), ord=1))
			if dist < 50:
				ok = False
		return ok

	def getColor(self):
		for i in range(1000):
			newColor = array([ri(0,255), ri(0,255), ri(0,255)])
			if self.colorCheck(newColor):
				break
		return newColor
		
	def open(self):
		print "Square: Square session opened!"
		self.handlers.append (self)
		self.color = self.getColor ()
		self.colors.append (self.color)
		self.write_message ('init ' + self.colorStr ())
		for handler in self.handlers:
			if handler != self:
				self.write_message ('create ' + handler.colorStr ())
		self.bcast ('create ' + self.colorStr ())
		self.printColors ()

	def on_message(self, message):
		self.bcast ('pos ' + message + ' ' + self.colorStr ())

	def on_close(self):
		self.bcast ('del ' + self.colorStr ())
		self.removeColor (self.color)
		self.handlers.remove (self)
		print "Square: Square session closed!"
		self.printColors ()
		
presents = []		
class PresentWS (WebSocketHandler):
	def open(self):
		print "Present: Present session opened!"
		presents.append (self)

	def on_close(self):
		presents.remove (self)
		print "Present: Present session closed!"
	
class RemoteWS (WebSocketHandler):
	def bcast (self, msg):
		for handler in presents:
			handler.write_message(msg)
				
	def open(self):
		print "Remote: Remote session opened!"

	def on_message(self, message):
		self.bcast (message)
		print "Remote: '" + message + "' broadcasted."

	def on_close(self):
		print "Remote: Remote session closed!"

application = tornado.web.Application([
	(r"/ws", SquareWS),
	(r"/remotews", RemoteWS),
	(r"/presentws", PresentWS),
	(r'/(.*)', tornado.web.StaticFileHandler, {'path': './static', 'default_filename': "index.html"}),
])

if __name__ == "__main__":
	application.listen(80)
	tornado.ioloop.IOLoop.instance().start()
